from max_ard.exceptions import MissingDependency
from shapely.geometry import shape
from shapely.ops import transform

from maxar_canvas_grid.grid import get_transform, get_CRS
from maxar_canvas_grid import covers

try:
    import rasterio
    from rasterio import windows
    HAS_RASTERIO = True
except ImportError:
    HAS_RASTERIO = False


        
def loop_geoms(collection, geoms, src_proj=4326):

        try:
            len(geoms)
        except TypeError:
            geoms = [geoms,]

        src_crs = get_CRS(src_proj)
        for geom in geoms:

            if type(geom) == dict:
                if 'geometry' in geom:
                        geom = geom['geometry']
                geom = shape(geom)

            for cell in covers(geom, src_proj=src_proj):
                for tile in collection.get_stack(cell):
                    tile_crs = get_CRS(tile.cell.proj)
                    tfm = get_transform(src_crs, tile_crs)
                    proj_geom = transform(tfm, geom)
                    yield tile, proj_geom

                

def read_windows(collection, geoms, src_proj=4326):
    ''' A generator for reading windows from overlapping tiles in an ARDCollection

            Args:
                geoms: A geometry object, or a list of geometries, which can be:
                    - shapely geometry objects
                    - dicts that follow the Python Geometry Interface (GeoJSON-like)
                    - GeoJSON feature-like dicts that have a 'geometry' key
                    - a Fiona dataset reader
                src_proj: An EPSG identifier parseable by Proj4

            Yields:
                Tuple of:
                    ARDTile of tile
                    Shapely polygon representing the bounds of the input geometry
                        in the tile's UTM coordinate system
                    a Reader function:

                        reader(asset)

                            Args:
                                asset(str): asset to read: `visual`, `pan`, or `ms`
                            
                            Returns:
                                numpy array of data covered by the returned geometry

            Example:

            geoms = [...]
            for tile, geom, reader in read_windows(geoms):
                # can check tile.properties, or fetch tile.clouds
                # can check if `geom` intersects with clouds
                data = reader('visual') '''

    if not HAS_RASTERIO:
        raise MissingDependency('Rasterio is required to read ARD rasters')

    for tile, geom in loop_geoms(collection, geoms, src_proj):

        def reader(asset):
            dsr = tile.open_asset(asset)
            window = rasterio.windows.from_bounds(*geom.bounds, dsr.transform)
            return dsr.read(window=window)

        yield (tile, geom, reader)


def write_windows(collection, geoms, src_proj=4326):

    if not HAS_RASTERIO:
        raise MissingDependency('Rasterio is required to read ARD rasters')

    for tile, geom in loop_geoms(collection, geoms, src_proj):
    
        def writer(asset, name, **kwargs):
            dsr = tile.open_asset(asset)
            window = rasterio.windows.from_bounds(*geom.bounds, dsr.transform)
            window = windows.from_bounds(*geom.bounds, dsr.transform)
            profile = dsr.meta
            profile.update({
                'width': window.width,
                'height': window.height,
                'count': dsr.count,
                'transform': dsr.window_transform(window),
                'driver': 'GTiff'
            })
            profile.update(kwargs)
            with rasterio.open(name, 'w', **profile) as dest:
                dest.write(dsr.read(window=window))

        yield (tile, geom, writer)